@@ -753,6 +753,14 @@ pub(crate) fn on_response(
             Ok(())
         }
         "textDocument/rename" => {
+            if let JsonValue::Null = result {
+                ctx.editor
+                    .status_bar
+                    .write(MessageKind::Error)
+                    .str("could not rename item under cursor");
+                return Ok(());
+            }
+
             let edit = WorkspaceEdit::from_json(result, &client.json)?;
             edit.apply(
                 &mut ctx.editor,
@@ -886,8 +894,8 @@ pub(crate) fn on_response(
             Ok(())
         }
         "textDocument/formatting" => {
-            let buffer_view_handle = match client.request_state {
-                RequestState::Formatting { buffer_view_handle } => buffer_view_handle,
+            let buffer_handle = match client.request_state {
+                RequestState::Formatting { buffer_handle } => buffer_handle,
                 _ => return Ok(()),
             };
             client.request_state = RequestState::Idle;
@@ -896,10 +904,6 @@ pub(crate) fn on_response(
                 _ => return Ok(()),
             };
 
-            let buffer_view = ctx.editor.buffer_views.get(buffer_view_handle);
-            let buffer_handle = buffer_view.buffer_handle;
-            let position = buffer_view.cursors.main_cursor().position;
-
             TextEdit::apply_edits(
                 &mut ctx.editor,
                 buffer_handle,
@@ -908,15 +912,16 @@ pub(crate) fn on_response(
                 &client.json,
             );
 
-            ctx.trigger_event_handlers();
-
-            let buffer_view = ctx.editor.buffer_views.get_mut(buffer_view_handle);
-            let mut cursors = buffer_view.cursors.mut_guard();
-            cursors.clear();
-            cursors.add(Cursor {
-                anchor: position,
-                position,
-            });
+            for buffer_view in ctx.editor.buffer_views.iter() {
+                let position = buffer_view.cursors.main_cursor().position;
+                ctx.editor.events.enqueue_fix_cursors(
+                    buffer_view.handle(),
+                    &[Cursor {
+                        anchor: position,
+                        position,
+                    }],
+                );
+            }
 
             Ok(())
         }
@@ -1079,4 +1084,3 @@ fn goto_definition(
         DefinitionLocation::Invalid => Ok(()),
     }
 }
-
